const chainIds = {
  hazama: 1,
  ethereum: 1,
  rinkeby: 4,
  goerli: 5,
  polygon: 137,
  mumbai: 80001,
  astar: 592,
  zkatana: 1261120,
  shiden: 336,
  celo: 42220,
  'celo-alfajores-testnet': 44787,
  binance: 56,
  bnb: 56,
  'binance-testnet': 97,
  avalanche: 43114,
  'avalanche-fuji': 43113,
  'polygon-zkevm': 1101,
  'polygon-zkevm-testnet': 1442,
}

exports.chainIds = chainIds

exports.SUPPORTED_CHAIN_NAME = [
  'ethereum',
  'goerli',
  'polygon',
  'mumbai',
  'avalanche-fuji',
  'celo-alfajores-testnet',
  'astar',
  'zkatana',
  'shiden',
  'celo',
  'binance',
  'binance-testnet',
  'avalanche',
  'polygon-zkevm',
  'polygon-zkevm-testnet',
]

exports.ALCHEMY_SUPPORTED_CHAIN_NAME = [
  'ethereum',
  'goerli',
  'polygon',
  'mumbai',
  'astar',
]

exports.WaitingAlchemyPeriod = 120000

exports.AlchemyApiKeys = Object.freeze({
  goerli: 'td3dNUbdRxiWjq40dzROhGQnEs5t_wQM',
  ethereum: '7XkQ3F7K3sdroSJonHDcTYA6cs1i3VYN',
  polygon: 'k81asPDI6lu1XyXhjpEs7Xz7vVTT-1wQ',
  mumbai: 'OHyYiQUHvfY4VSi5CfP1nkt-E08R1jMC',
  astar: 'zNw-OoUjQxIWhGb3E_58g1DNLNB-ans2',
})

exports.AlchemyNetworks = Object.freeze({
  goerli: 'eth-goerli',
  ethereum: 'eth-mainnet',
  polygon: 'polygon-mainnet',
  mumbai: 'polygon-mumbai',
  astar: 'astar-mainnet',
})

exports.NetworkInfo = Object.freeze({
  hazama: {
    chainId: `0x${chainIds.hazama.toString(16)}`,
    chainName: 'Hazama Network',
    name: 'Hazama',
    rpcUrls: ['https://mainnet.infura.io/v3/9aa3d95b3bc440fa88ea12eaa4456161'],
  },
  ethereum: {
    chainId: `0x${chainIds.ethereum.toString(16)}`,
    chainName: 'Ethereum Network',
    name: 'Ethereum',
    nativeCurrency: {
      name: 'ETH',
      symbol: 'ETH',
      decimals: 18,
    },
    rpcUrls: [
      `https://${exports.AlchemyNetworks.ethereum}.g.alchemy.com/v2/${exports.AlchemyApiKeys.ethereum}`,
    ],
  },
  polygon: {
    chainId: `0x${chainIds.polygon.toString(16)}`,
    chainName: 'Polygon Network',
    name: 'Polygon',
    nativeCurrency: {
      name: 'MATIC',
      symbol: 'MATIC',
      decimals: 18,
    },
    rpcUrls: [
      `https://${exports.AlchemyNetworks.polygon}.g.alchemy.com/v2/${exports.AlchemyApiKeys.polygon}`,
    ],
  },
  astar: {
    chainId: `0x${chainIds.astar.toString(16)}`,
    chainName: 'Astar Network',
    name: 'Astar',
    nativeCurrency: {
      name: 'ASTR',
      symbol: 'ASTR',
      decimals: 18,
    },
    rpcUrls: ['https://rpc.astar.network:8545'],
    blockExplorerUrls: ['https://astar.subscan.io/'],
  },
  zkatana: {
    chainId: `0x${chainIds.zkatana.toString(16)}`,
    chainName: 'zKatana',
    name: 'zKatana',
    nativeCurrency: {
      name: 'Sepolia Ether (ETH)',
      symbol: 'ETH',
      decimals: 18,
    },
    rpcUrls: ['https://rpc.startale.com/zkatana'],
    blockExplorerUrls: ['https://zkatana.blockscout.com/'],
  },
  goerli: {
    chainId: `0x${chainIds.goerli.toString(16)}`,
    chainName: 'Goerli Network',
    name: 'Goerli',
    nativeCurrency: {
      name: 'ETH',
      symbol: 'ETH',
      decimals: 18,
    },
    rpcUrls: [
      `https://${exports.AlchemyNetworks.goerli}.g.alchemy.com/v2/${exports.AlchemyApiKeys.goerli}`,
    ],
  },
  mumbai: {
    chainId: `0x${chainIds.mumbai.toString(16)}`,
    chainName: 'Mumbai Network',
    name: 'Mumbai',
    nativeCurrency: {
      name: 'MATIC',
      symbol: 'MATIC',
      decimals: 18,
    },
    rpcUrls: [
      `https://${exports.AlchemyNetworks.mumbai}.g.alchemy.com/v2/${exports.AlchemyApiKeys.mumbai}`,
    ],
  },
  shiden: {
    chainId: `0x${chainIds.shiden.toString(16)}`,
    chainName: 'Shiden Network',
    name: 'Shiden',
    nativeCurrency: {
      name: 'Shiden',
      symbol: 'SDN',
      decimals: 18,
    },
    rpcUrls: [
      'https://shiden.api.onfinality.io/public',
      'https://shiden-rpc.dwellir.com',
      'https://shiden.public.blastapi.io',
    ],
  },
  celo: {
    chainId: `0x${chainIds.celo.toString(16)}`,
    chainName: 'Celo Mainnet Network',
    name: 'Celo Mainnet',
    nativeCurrency: {
      name: 'CELO',
      symbol: 'CELO',
      decimals: 18,
    },
    rpcUrls: ['https://forno.celo.org'],
  },
  'celo-alfajores-testnet': {
    chainId: `0x${chainIds['celo-alfajores-testnet'].toString(16)}`,
    chainName: 'Celo Alfajores Testnet Network',
    name: 'Celo Alfajores Testnet',
    nativeCurrency: {
      name: 'CELO',
      symbol: 'CELO',
      decimals: 18,
    },
    rpcUrls: ['https://alfajores-forno.celo-testnet.org'],
  },
  binance: {
    chainId: `0x${chainIds.binance.toString(16)}`,
    chainName: 'BNB Smart Chain Mainnet Network',
    name: 'BNB Smart Chain Mainnet',
    nativeCurrency: {
      name: 'BNB Chain Native Token',
      symbol: 'BNB',
      decimals: 18,
    },
    rpcUrls: [
      'https://bsc.publicnode.com',
      'https://bsc-dataseed4.ninicoin.io',
      'https://bsc-dataseed3.ninicoin.io',
      'https://bsc-dataseed2.ninicoin.io',
      'https://bsc-dataseed1.ninicoin.io',
      'https://bsc-dataseed4.defibit.io',
      'https://bsc-dataseed3.defibit.io',
      'https://bsc-dataseed2.defibit.io',
      'https://bsc-dataseed1.defibit.io',
      'https://bsc-dataseed4.bnbchain.org',
      'https://bsc-dataseed3.bnbchain.org',
      'https://bsc-dataseed2.bnbchain.org',
      'https://bsc-dataseed1.bnbchain.org',
    ],
  },
  'binance-testnet': {
    chainId: `0x${chainIds['binance-testnet'].toString(16)}`,
    chainName: 'BNB Smart Chain Testnet Network',
    name: 'BNB Smart Chain Testnet',
    nativeCurrency: {
      name: 'BNB Chain Native Token',
      symbol: 'tBNB',
      decimals: 18,
    },
    rpcUrls: [
      'https://bsc-testnet.publicnode.com',
      'https://data-seed-prebsc-2-s3.bnbchain.org:8545',
      'https://data-seed-prebsc-1-s3.bnbchain.org:8545',
      'https://data-seed-prebsc-2-s2.bnbchain.org:8545',
      'https://data-seed-prebsc-1-s2.bnbchain.org:8545',
      'https://data-seed-prebsc-2-s1.bnbchain.org:8545',
      'https://data-seed-prebsc-1-s1.bnbchain.org:8545',
    ],
  },
  avalanche: {
    chainId: `0x${chainIds.avalanche.toString(16)}`,
    chainName: 'Avalanche C-Chain Network',
    name: 'Avalanche C-Chain',
    nativeCurrency: {
      name: 'Avalanche',
      symbol: 'AVAX',
      decimals: 18,
    },
    rpcUrls: [
      'https://api.avax.network/ext/bc/C/rpc',
      'https://avalanche-c-chain.publicnode.com',
    ],
  },
  'avalanche-fuji': {
    chainId: `0x${chainIds['avalanche-fuji'].toString(16)}`,
    chainName: 'Avalanche Fuji Testnet Network',
    name: 'Avalanche Fuji Testnet',
    nativeCurrency: {
      name: 'Avalanche',
      symbol: 'AVAX',
      decimals: 18,
    },
    rpcUrls: [
      'https://api.avax-test.network/ext/bc/C/rpc',
      'https://avalanche-fuji-c-chain.publicnode.com',
    ],
  },
  'polygon-zkevm': {
    chainId: `0x${chainIds['polygon-zkevm'].toString(16)}`,
    chainName: 'Polygon zkEVM Network',
    name: 'Polygon zkEVM',
    nativeCurrency: {
      name: 'Ether',
      symbol: 'ETH',
      decimals: 18,
    },
    rpcUrls: ['https://zkevm-rpc.com'],
  },
  'polygon-zkevm-testnet': {
    chainId: `0x${chainIds['polygon-zkevm-testnet'].toString(16)}`,
    chainName: 'Polygon zkEVM Testnet Network',
    name: 'Polygon zkEVM Testnet',
    nativeCurrency: {
      name: 'Ether',
      symbol: 'ETH',
      decimals: 18,
    },
    rpcUrls: ['https://rpc.public.zkevm-test.net'],
  },
})

/**
 *
 * @private
 * @param {string} network network
 * @returns {string}
 */
exports.getRpcUrlsByNetWork = (network) => {
  return network.startsWith('http') || network.startsWith('ws')
    ? network
    : exports.NetworkInfo[network]?.rpcUrls?.[0] || ''
}

/**
 *
 * @private
 * @param {string} network network
 * @returns {string}
 */
exports.getAlchemyApiKey = (network) => {
  return exports.AlchemyApiKeys[network] || ''
}

/**
 *
 * @private
 * @param {string} network network
 * @returns {string}
 */
exports.getAlchemyNetwork = (network) => {
  return exports.AlchemyNetworks[network] || ''
}

/**
 *
 * @private
 * @param {string} network network
 * @returns {number | null}
 */
exports.getChainId = (network) => {
  return chainIds?.[network] || null
}

exports.HazamaHost = 'wss://n.hazbase.com'
